<?php

namespace Laravel\Laravel\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Laravel\Laravel\Http\Requests\HubRequest;
use Laravel\Laravel\Interfaces\HubRepositoryInterface;
use Laravel\Laravel\Models\Hub;

/**
 * Resource controller class for hub.
 */
class HubResourceController extends BaseController
{

    /**
     * Initialize hub resource controller.
     *
     * @param type HubRepositoryInterface $hub
     *
     * @return null
     */
    public function __construct(HubRepositoryInterface $hub)
    {
        parent::__construct();
        $this->repository = $hub;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Laravel\Laravel\Repositories\Criteria\HubResourceCriteria::class);
    }

    /**
     * Display a list of hub.
     *
     * @return Response
     */
    public function index(HubRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Laravel\Laravel\Repositories\Presenter\HubPresenter::class)
                ->$function();
        }

        $hubs = $this->repository->paginate();

        return $this->response->title(trans('laravel::hub.names'))
            ->view('laravel::hub.index', true)
            ->data(compact('hubs'))
            ->output();
    }

    /**
     * Display hub.
     *
     * @param Request $request
     * @param Model   $hub
     *
     * @return Response
     */
    public function show(HubRequest $request, Hub $hub)
    {

        if ($hub->exists) {
            $view = 'laravel::hub.show';
        } else {
            $view = 'laravel::hub.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('laravel::hub.name'))
            ->data(compact('hub'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new hub.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(HubRequest $request)
    {

        $hub = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('laravel::hub.name')) 
            ->view('laravel::hub.create', true) 
            ->data(compact('hub'))
            ->output();
    }

    /**
     * Create new hub.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(HubRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $hub                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('laravel::hub.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('laravel/hub/' . $hub->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/laravel/hub'))
                ->redirect();
        }

    }

    /**
     * Show hub for editing.
     *
     * @param Request $request
     * @param Model   $hub
     *
     * @return Response
     */
    public function edit(HubRequest $request, Hub $hub)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('laravel::hub.name'))
            ->view('laravel::hub.edit', true)
            ->data(compact('hub'))
            ->output();
    }

    /**
     * Update the hub.
     *
     * @param Request $request
     * @param Model   $hub
     *
     * @return Response
     */
    public function update(HubRequest $request, Hub $hub)
    {
        try {
            $attributes = $request->all();

            $hub->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('laravel::hub.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('laravel/hub/' . $hub->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('laravel/hub/' . $hub->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the hub.
     *
     * @param Model   $hub
     *
     * @return Response
     */
    public function destroy(HubRequest $request, Hub $hub)
    {
        try {

            $hub->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('laravel::hub.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('laravel/hub/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('laravel/hub/' . $hub->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple hub.
     *
     * @param Model   $hub
     *
     * @return Response
     */
    public function delete(HubRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('laravel::hub.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('laravel/hub'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/laravel/hub'))
                ->redirect();
        }

    }

    /**
     * Restore deleted hubs.
     *
     * @param Model   $hub
     *
     * @return Response
     */
    public function restore(HubRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('laravel::hub.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/laravel/hub'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/laravel/hub/'))
                ->redirect();
        }

    }

}
